@@ -5,14 +5,15 @@ use warnings;
 use Carp qw/croak/;
 use XSLoader;
 
-our $VERSION = '0.36'; # Don't forget to update the TestCompat set for testing against installed encoders!
+our $VERSION = '2.00_01'; # Don't forget to update the TestCompat set for testing against installed encoders!
 
 # not for public consumption, just for testing.
-my $TestCompat = [ map sprintf("%.2f", $_/100), reverse( 23 .. int($VERSION * 100) ) ]; # compat with 0.23 to ...
+(my $num_version = $VERSION) =~ s/_//;
+my $TestCompat = [ map sprintf("%.2f", $_/100), reverse( 200 .. int($num_version * 100) ) ]; # compat with 0.23 to ...
 sub _test_compat {return(@$TestCompat, $VERSION)}
 
 use Exporter 'import';
-our @EXPORT_OK = qw(decode_sereal looks_like_sereal);
+our @EXPORT_OK = qw(decode_sereal looks_like_sereal decode_sereal_with_header_data);
 our %EXPORT_TAGS = (all => \@EXPORT_OK);
 # export by default if run from command line
 our @EXPORT = ((caller())[1] eq '-e' ? @EXPORT_OK : ());
@@ -54,19 +55,16 @@ Sereal::Decoder - Fast, compact, powerful binary deserialization
 
 =head1 DESCRIPTION
 
-B<This is an experimental module. The interface may change without notice.
-Before using it in production, please get in touch with the authors!>
-
 This library implements a deserializer for an efficient, compact-output,
 and feature-rich binary protocol called I<Sereal>.
 Its sister module L<Sereal::Encoder> implements an encoder for this format.
 The two are released separately to allow for independent and safer upgrading.
 
-The Sereal protocol version that is compatible with this decoder implementation
-is currently protocol version 1. As it stands, it will refuse to attempt to
-decode future versions of the protocol, but there is likely going to be an
-option to decode the parts of the input that are compatible with version 1
-of the protocol. The protocol was designed to allow for this.
+The Sereal protocol versions that are compatible with this decoder implementation
+are currently protocol versions 1 and 2. As it stands, it will refuse to attempt to
+decode future versions of the protocol, but if necessary there is likely
+going to be an option to decode the parts of the input that are compatible
+with version 2 of the protocol. The protocol was designed to allow for this.
 
 The protocol specification and many other bits of documentation
 can be found in the github repository. Right now, the specification is at
@@ -275,7 +273,20 @@ C<Sereal::Decoder> objects will become a reference to undef in the new
 thread. This might change in a future release to become a full clone
 of the decoder object.
 
-=head1 AUTHOR
+=head1 BUGS, CONTACT AND SUPPORT
+
+For reporting bugs, please use the github bug tracker at
+L<http://github.com/Sereal/Sereal/issues>.
+
+For support and discussion of Sereal, there are two Google Groups:
+
+Announcements around Sereal (extremely low volume):
+L<https://groups.google.com/forum/?fromgroups#!forum/sereal-announce>
+
+Sereal development list:
+L<https://groups.google.com/forum/?fromgroups#!forum/sereal-dev>
+
+=head1 AUTHORS
 
 Yves Orton E<lt>demerphq@gmail.comE<gt>
 
